package xyz.tbvns.Inputs;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g3d.Material;
import com.badlogic.gdx.graphics.g3d.Model;
import com.badlogic.gdx.graphics.g3d.ModelInstance;
import com.badlogic.gdx.graphics.g3d.utils.CameraInputController;
import com.badlogic.gdx.graphics.g3d.utils.MeshPartBuilder;
import com.badlogic.gdx.graphics.g3d.utils.ModelBuilder;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.Octree;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.math.collision.BoundingBox;
import com.badlogic.gdx.math.collision.Ray;
import com.badlogic.gdx.physics.bullet.collision.btCollisionObject;
import xyz.tbvns.Editor.ModelsManager;
import xyz.tbvns.Main;
import xyz.tbvns.Physics.BulletManager;
import xyz.tbvns.ui.UIManager;
import xyz.tbvns.ui.Windows.ObjectEdit;

public class inputManager extends CameraInputController {
    private float startX, startY;
    private final Vector3 tmpV1 = new Vector3();
    private final Vector3 tmpV2 = new Vector3();

    public inputManager(Camera camera) {
        super(camera);
        rotateAngle = 100;
    }

    @Override
    public boolean scrolled(float amountX, float amountY) {
        camera.position.add(-camera.direction.x * amountY, -camera.direction.y * amountY, -camera.direction.z * amountY);
        target = camera.position;
        camera.update();
        return true;
    }

    @Override
    protected boolean process (float deltaX, float deltaY, int button) {
        if (button == rotateButton) {
            tmpV1.set(camera.direction).crs(camera.up).y = 0f;
            camera.rotateAround(target, tmpV1.nor(), deltaY * rotateAngle);
            camera.rotateAround(target, Vector3.Y, deltaX * -rotateAngle);

        } else if (button == translateButton) {
            camera.translate(tmpV1.set(camera.direction).crs(camera.up).nor().scl(-deltaX * translateUnits));
            camera.translate(tmpV2.set(camera.up).scl(-deltaY * translateUnits));
            if (translateTarget) target.add(tmpV1).add(tmpV2);
        } else if (button == forwardButton) {
            camera.translate(tmpV1.set(camera.direction).scl(deltaY * translateUnits));
            if (forwardTarget) target.add(tmpV1);
        }
        if (autoUpdate) camera.update();
        return true;
    }

    @Override
    public boolean touchDown(int screenX, int screenY, int pointer, int button) {
        if (button == Input.Buttons.LEFT || button == Input.Buttons.RIGHT ) {
            Gdx.input.setCursorCatched(true);
        }

        return super.touchDown(screenX, screenY, pointer, button);
    }

    @Override
    public boolean touchUp(int screenX, int screenY, int pointer, int button) {
        if (button == Input.Buttons.LEFT || button == Input.Buttons.RIGHT ) {
            Gdx.input.setCursorCatched(false);
        }

        return super.touchUp(screenX, screenY, pointer, button);
    }

    @Override
    public boolean touchDown(float x, float y, int pointer, int button) {
        x-=200;

        Ray ray = Main.getCamera().getPickRay(x, y, 0, 200, Main.getCamera().viewportWidth, Main.getCamera().viewportHeight);

        Vector3 rayStart = ray.origin;
        Vector3 rayEnd = ray.origin.cpy().add(ray.direction.cpy().scl(1000f));  // Long ray to ensure it reaches objects

        ray.set(rayStart, rayEnd);

        int hit = 0;

        for (ModelInstance model : ModelsManager.getLoadedModels()) {
            BoundingBox boundingBox = new BoundingBox();
            model.calculateBoundingBox(boundingBox);
            Vector3 pos = model.transform.getTranslation(Vector3.Zero);

            Vector3 min = new Vector3(pos.x + boundingBox.min.x, pos.y + boundingBox.min.y, pos.z + + boundingBox.min.z);
            Vector3 max = new Vector3(pos.x + boundingBox.max.x, pos.y + boundingBox.max.y, pos.z + + boundingBox.max.z);

            boundingBox.set(min, max);
            if (Intersector.intersectRayBoundsFast(ray, boundingBox)) {
                ModelsManager.select(model);
                UIManager.getActiveUI().put(ObjectEdit.class, true);
                hit++;
            }
        }

        if (hit == 0) {
            ModelsManager.deselect();
            UIManager.getActiveUI().put(ObjectEdit.class, false);
        }
//        Uncomment for debug line for raycast
//        ModelBuilder modelBuilder = new ModelBuilder();
//        modelBuilder.begin();
//        MeshPartBuilder builder = modelBuilder.part("line", 1, 3, new Material());
//        builder.setColor(Color.RED);
//        builder.line(rayStart, rayEnd);
//        Model lineModel = modelBuilder.end();
//        Main.LineModel = lineModel;

//        This doesn't work for some reason.
//        TODO: Make this work

//        btCollisionObject object = BulletManager.raycast(rayStart, rayEnd);
//        if (object != null) {
//            BulletManager.getModelCollisiontHashMap().forEach((k, v) -> {
//                if (v == object) {
//                    ModelsManager.select(k);
//                    UIManager.getActiveUI().put(ObjectEdit.class, true);
//                }
//            });
//        } else {
//            if (ModelsManager.getSelectedModelInstance() != null) {
//                ModelsManager.deselect();
//            }
//        }

        return super.touchDown(x, y, pointer, button);
    }
}
